// file:   obj/webster.c
// author: Truilkan@Basis
// date:   1992/10/29
// mudlib: Basis

// if you choose to use this code in your mud, please retain the above
// header.  if you write a help screen for this object, please give credit
// to the original authors (Truilkan and Jacques)

// This object obeys part of the RFC for dictionary lookups.  It provides
// an LPC object frontend (in the form of a dictionary object) to an online
// dictionary server.

// todo: parse the first line returned in order to be smart about
// interpreting results of a query (successful, failed, etc).

#include <config.h>
#include <daemons.h>
#include <socket.h>
#include <attributes.h>

#define DISCONNECTED "a dictionary (closed)"
#define CONNECTED "a dictionary (open)"

inherit BASE;
inherit "/std/socket/telnet";

void
create()
{
        set_author("truilkan");
	base::create();
	telnet::create();
	set(a_ids, ({"dictionary", "book", "webster"}) );
	set(a_eshort, DISCONNECTED);
	set(a_elong, "Its a fine dictionary with faded gold embossing.\n"
	+ "You could probably do all manner of things with it such as defining,\n"
	+ "completing, and spelling words.\n");
	set_verbosity(0);
}

void
handler(string event)
{
	switch (event) {
		case "open" :
		set(a_eshort, CONNECTED);
		tell_object(environment(this_object()),
			"The dictionary creaks open.\n");
		break;
		case "close" :
		tell_object(environment(this_object()),
			"The dictionary slams shut!\n");
		set(a_eshort, DISCONNECTED);
		break;
		default:
		break;
	}
}

int
dlookup(string arg)
{
	int result;

	if (!query_connected()) {
		notify_fail("The dictionary isn't open!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " looks up a word.\n");
	telnet::send("DEFINE " + arg + "\n");
	return 1;
}

int
dopen(string arg)
{
	if (query_connected()) {
		notify_fail("It's already open!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " opens "
		+ possessive(this_player()->query(a_gender)) + " dictionary.\n");
	return telnet::connect("129.79.254.191 2627");
}

int
dclose(string arg)
{
	if (!query_connected()) {
		notify_fail("It's already closed!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " closes "
		+ possessive(this_player()->query(a_gender)) + " dictionary.\n");
	return telnet::disconnect(arg);
}

int
dskim(string arg)
{
	int result;

	if (!query_connected()) {
		notify_fail("The dictionary isn't open!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " skims the dictionary.\n");
	telnet::send("ENDINGS " + arg + "\n");
	return 1;
}

int
dspell(string arg)
{
	int result;

	if (!query_connected()) {
		notify_fail("The dictionary isn't open!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " searches for a word.\n");
	telnet::send("SPELL " + arg + "\n");
	return 1;
}

int
dcomplete(string arg)
{
	int result;

	if (!query_connected()) {
		notify_fail("The dictionary isn't open!\n");
		return 0;
	}
	say((string)this_player()->query(a_cap_name) + " searches for a word.\n");
	telnet::send("COMPLETE " + arg + "\n");
	return 1;
}

void
init()
{
	add_action("dlookup", "define");
	add_action("dskim", "endings");
	add_action("dopen", "open");
	add_action("dclose", "close");
	add_action("dspell", "spell");
	add_action("dcomplete", "complete");
}
