/*
   queue.c: simple fixed-size queue (FIFO) in LPC 3.0
    -- by Truilkan@TMI - 92/01/31
    -- modified 92/10/17 -- Truilkan
*/

private mixed *queue;
private int hptr, tptr, size;
private int count;
private object owner;

mixed *
query_queue()
{
   if (previous_object() != owner) {
      return ({});
   } 
   return queue;
}

mixed dequeue()
{
   int h;

   if (previous_object() != owner) {
      return -1;
   } 
   if (!count) {
      return -1;
   }
   count--;
   h = hptr;
   hptr = (hptr + 1) % size;
   return queue[h];          /* pre-increment */
}

int enqueue(mixed elt)
{
   if (previous_object() != owner) {
      return -1;
   } 
   if (count++ == size) {
      return -1;
   }
   queue[tptr] = elt;  /* post-increment */
   tptr = (tptr + 1) % size;
   return 0;
}

void alloc(int s)
{
   count = 0;
   hptr = 0;
   tptr = 0;
   size = s;
   owner = previous_object();
   queue = allocate(size);
}

mixed access(int i)
{
   int j;

   if (i < 0) {
      return 0;
   }
   j = (hptr + i) % size;
   if (j >= 0 && j < size) {
      return queue[j];
   } else {
      return 0;
   }
}

void remove()
{
   if (previous_object() != owner) {
      return;
   } 
   destruct(this_object());
}
