//  mudlib:  Basis
//  file:    core.c
//  author:  Truilkan
//  created: 1992/09/08
//  purpose: implements the core functions for setting and querying variables.
//   The intent is for this object to be inheritable by any object that
//   has a need to respond to set() and query().  The idea is to provide
//   a base level of functionality without providing (most) other unneeded
//   methods.

/*
  $Locker:  $

  $Source: /usr/local/mud/libs/basis/std/RCS/core.c,v $
  $Revision: 1.2 $
  $Author: garnett $
  $Date: 92/09/24 19:42:31 $
  $State: Exp $

  $Log:	core.c,v $
 * Revision 1.2  92/09/24  19:42:31  garnett
 * abstracted out the save code into /std/save.c
 * 
 * Revision 1.1  92/09/24  06:12:26  garnett
 * Initial revision
 * 
*/

#include <config.h>
#include <daemons.h>
inherit SAVE;

// private attrs since all access to attrs should go through the accessor
// functions in this object.

private mapping attrs;

// set_persistent only callable by this_object() (including inheritors)

void
delete(mixed key)
{
	map_delete(attrs, key);
}

void
set(mixed key, mixed value)
{
    attrs[key] = value;
}

mixed
query(mixed key)
{
    return attrs[key];
}

void
add(mixed key, mixed value)
{
	if (undefinedp(attrs[key])) {
		set(key, value);
		return;
	}
	// don't allow (x + -y) as a synonum for (x - y)
	// (x + -y) conducive to coding errors (money bugs etc.)
	if (intp(value) && (value < 0)) {
		return;
	}
	attrs[key] += value;
}

void
subtract(mixed key, mixed value)
{
	if (undefinedp(attrs[key])) {
		return;
	}
	// don't allow (x - (-y)) as a synonym for (x + y)
	// (x - (-y)) conducive to coding errors (money bugs etc.)
	if (intp(value) && (value < 0)) {
		return;
	}
	attrs[key] -= value;
}

void
remove()
{
	save::remove();
}

void
create()
{
	attrs = ([]);
	save::create();
}
